package dk.jkkn.util;

import java.io.File;
import java.io.IOException;
import java.net.JarURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;

/**
 * Utility class to abstract away differences in looking at resources versa files
 * 
 * @author Kristian Krmmer Nielsen <jkkn@jkkn.dk>
 */
public class Resources {

	/**
	 * Creates a directory listing for the given URL path Handles either normal files and
	 * jar-packages
	 * 
	 * @param path Path to create a directory listing for
	 * @param filter Filter to use
	 * @return List of files in the given directory
	 * @throws IOException in case of a read error
	 */
	public static List<URL> getDirectoryList(final URL path, final ResourceFilter filter)
			throws IOException {
		final List<URL> files = new ArrayList<URL>();

		if (path != null) {

			if (path.getProtocol().equals("file")) {

				final File file = new File(path.getFile());
				if (file.isDirectory() && file.canRead()) {
					for (final File f : file.listFiles()) {
						final URL url = f.toURI().toURL();
						final ResourceWrap wrap = new ResourceWrap(url, f.isDirectory(),
								f.isFile(), f.canRead());

						if (filter.accept(wrap)) {
							files.add(url);
						}
					}
				}

			} else if (path.getProtocol().equals("jar")) {
				final JarURLConnection jar = (JarURLConnection) path.openConnection();
				final URL jarURL = new URL("jar:" + jar.getJarFileURL().toExternalForm()
						+ "!/");
				String entryFilter = jar.getEntryName();
				if (entryFilter != null) {
					if (!entryFilter.endsWith("/")) {
						entryFilter = entryFilter + "/";
					}
				} else {
					entryFilter = "";
				}

				for (final Enumeration<JarEntry> entries = jar.getJarFile().entries(); entries
						.hasMoreElements();) {
					final JarEntry entry = entries.nextElement();
					final String entryName = entry.getName();

					/* filter manually parts of the jar file */
					if (entryName.length() <= entryFilter.length()) {
						continue;
					}

					/* cut out path name */
					final int pathEnd = entryName
							.lastIndexOf("/", entryName.length() - 2);
					if (pathEnd == -1 && entryFilter.length() > 0) {
						continue;
					}
					if (!entryName.substring(0, pathEnd + 1).equals(entryFilter)) {
						continue;
					}

					if (entryName.startsWith(entryFilter)) {
						final URL url = new URL(jarURL, entry.getName());
						final ResourceWrap wrap = new ResourceWrap(url, entry
								.isDirectory(), !entry.isDirectory(), true);

						if (filter.accept(wrap)) {
							files.add(url);
						}
					}
				}

			} else {
				throw new IOException("Unable to handle protocol: " + path.getProtocol());
			}
		}

		return files;
	}

}
