package dk.sdu.imada.jkkn04.Protein.data;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import dk.jkkn.util.ResourceFilter;
import dk.jkkn.util.ResourceWrap;
import dk.jkkn.util.Resources;
import dk.sdu.imada.jkkn04.Protein.data.Protein.TranslationType;

/**
 * Represents a complete collection of proteins
 * 
 * @author Kristian Krmmer Nielsen <jkkn@jkkn.dk>
 */
public class ProteinMapCollection {

	/**
	 * The release bundle some example data, this is the default location of these
	 */
	public final static URL DEFAULT_DATA_PATH = ProteinMapCollection.class
			.getResource("/data/protein/");

	final private HashMap<String, ProteinMap> collection = new HashMap<String, ProteinMap>();
	final private SortedSet<String> proteinIdentifiers = new TreeSet<String>();

	/**
	 * Creates a new collection and reads all files available in the given data directory.
	 * All files with the extensions ".pred" and ".fasta" is read.
	 * 
	 * @param dataDir directory to read
	 * @throws IOException in case of a read error.
	 */
	public ProteinMapCollection(final URL dataDir) throws IOException {
		final List<ProteinFile> proteinFiles = new ArrayList<ProteinFile>();
		for (final URL file : Resources.getDirectoryList(dataDir, new ResourceFilter() {

			@Override
			public boolean accept(final ResourceWrap file) {
				return file.isFile()
						&& file.canRead()
						&& (file.getName().endsWith(".pred") || file.getName().endsWith(
								".fasta"));
			}

		})) {
			proteinFiles.add(new ProteinFile(file));
		}

		for (final ProteinFile pf : proteinFiles) {
			/* Create a sorted set of all existing proteins */
			proteinIdentifiers.addAll(pf.getProteins().keySet());
			/* Create a map for every protein sequence type => protein */
			for (final Protein p : pf.getProteins().values()) {
				ProteinMap pm;
				if (!collection.containsKey(p.getIdentifier())) {
					collection.put(p.getIdentifier(), pm = new ProteinMap());
				} else {
					pm = collection.get(p.getIdentifier());
				}
				for (final Protein tp : Protein.getTranslatedProteins(p)) {
					pm.put(new SequenceTranslated(tp.getSequenceType(), tp
							.getTranslationType()), tp);
				}
			}
		}

		// Hardcoded reference type to be KAKSI:
		setProteinReferenceSequenceType(new SequenceTranslated(SequenceType.KAKSI,
				TranslationType.NORMAL));

	}

	/**
	 * Creates a new collection for a given directory
	 * 
	 * @param dataDir directory to read
	 * @throws IOException in case of a read error.
	 * @see #ProteinMapCollection(URL)
	 */
	public ProteinMapCollection(final String dataDir) throws IOException {
		this(new URL(DEFAULT_DATA_PATH, dataDir));
	}

	/**
	 * This determinds which type we should use as reference when reading the sequences.
	 * If unknowns are found in the reference, they will also be read as unknown in the
	 * other sequences.
	 * 
	 * @param type Type to use as reference
	 */
	public void setProteinReferenceSequenceType(final SequenceTranslated type) {
		/*
		 * If we have a reference sequence (KAKSI), set it so we can clean the protein
		 * sequence appropriately
		 */
		for (final ProteinMap pm : collection.values()) {
			if (pm.containsKey(type)) {
				final Protein reference = pm.get(type);
				for (final Map.Entry<SequenceTranslated, Protein> entry : pm.entrySet()) {
					if (!entry.getKey().equals(type)) {
						entry.getValue().setReferenceSequence(reference);
					}
				}
			} else {
				// warning
			}
		}

	}

	/**
	 * Returns a map of all sequence identifiers
	 * 
	 * @return map of ids
	 */
	public Set<String> getProteinIdentifiers() {
		return Collections.unmodifiableSet(proteinIdentifiers);
	}

	/**
	 * Return a map for a given protein identifier
	 * 
	 * @param identifier Protein identifier
	 * @return map
	 */
	public ProteinMap getProteinMap(final String identifier) {
		return collection.get(identifier);
	}

}
