package dk.sdu.imada.jkkn04.Protein.data;

import java.util.HashSet;
import java.util.Set;

import dk.sdu.imada.jkkn04.Protein.data.Protein.TranslationType;

/**
 * Represents a protein sequence type in a given translation form.
 * 
 * @author Kristian Krmmer Nielsen <jkkn@jkkn.dk>
 */
public class SequenceTranslated implements Comparable<SequenceTranslated> {

	/**
	 * Creates a new representative for a given protein type in a given translation
	 * 
	 * @param sequenceType Protein sequence type
	 * @param translationType Translation type
	 */
	public SequenceTranslated(final SequenceType sequenceType,
			final TranslationType translationType) {
		super();
		this.sequenceType = sequenceType;
		this.translationType = translationType;
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((sequenceType == null) ? 0 : sequenceType.hashCode());
		result = prime * result
				+ ((translationType == null) ? 0 : translationType.hashCode());
		return result;
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final SequenceTranslated other = (SequenceTranslated) obj;
		if (sequenceType == null) {
			if (other.sequenceType != null)
				return false;
		} else if (!sequenceType.equals(other.sequenceType))
			return false;
		if (translationType == null) {
			if (other.translationType != null)
				return false;
		} else if (!translationType.equals(other.translationType))
			return false;
		return true;
	}

	/**
	 * Returns name of sequence type appended with translation if not
	 * TranslationType.NORMAL
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return sequenceType.name()
				+ (translationType != TranslationType.NORMAL ? " "
						+ translationType.name() : "");
	}

	/**
	 * Return sequence type
	 * 
	 * @return sequence type
	 */
	public SequenceType getSequenceType() {
		return sequenceType;
	}

	/**
	 * Return translation type
	 * 
	 * @return type of translation
	 */
	public TranslationType getTranslationType() {
		return translationType;
	}

	private final SequenceType sequenceType;
	private final TranslationType translationType;

	/**
	 * Construct a name for this sequencetype/translation pair.
	 * 
	 * @return type + ":" + translation
	 */
	public String name() {
		return sequenceType.name() + ":" + translationType;
	}

	/**
	 * Creates a set of all possible translations of a given set of sequence types
	 * 
	 * @param types sequence types
	 * @return set of all translation of those sequence types
	 */
	public static Set<SequenceTranslated> buildAllOf(final Set<SequenceType> types) {
		final Set<SequenceTranslated> set = new HashSet<SequenceTranslated>();
		for (final SequenceType type : types) {
			for (final TranslationType transType : TranslationType.values()) {
				set.add(new SequenceTranslated(type, transType));
			}
		}
		return set;
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(final SequenceTranslated o) {
		if (o.sequenceType != sequenceType)
			return sequenceType.compareTo(o.sequenceType);
		return translationType.compareTo(o.translationType);
	}

}
