/**
 * 
 */
package dk.sdu.imada.jkkn04.Protein.data;

import java.util.EnumSet;

/**
 * Represents an observed or predicted sequence type
 * 
 * @author Kristian Krmmer Nielsen <jkkn@jkkn.dk>
 */
public enum SequenceType {
	/**
	 * Primary sequence string
	 */
	PRIMARY(),
	/**
	 * DSSP type observation sequence
	 */
	DSSP(EnumSet.of(SequenceAttributes.OBSERVATION, SequenceAttributes.EXTENDED_CHARS)),
	/**
	 * STRIDE type observation sequence
	 */
	STRIDE(EnumSet.of(SequenceAttributes.OBSERVATION, SequenceAttributes.EXTENDED_CHARS)),
	/**
	 * KAKSI type observation sequence
	 */
	KAKSI(EnumSet.of(SequenceAttributes.OBSERVATION)),
	/**
	 * SVM type prediction sequence
	 */
	SVM_PRED(EnumSet.of(SequenceAttributes.PREDICTION)),
	/**
	 * SVM VITERBI type prediction sequence
	 */
	SVM_VITERBI(EnumSet.of(SequenceAttributes.PREDICTION)),
	/**
	 * BRNN type prediction sequence
	 */
	BRNN_PRED(EnumSet.of(SequenceAttributes.PREDICTION)),
	/**
	 * BRNN VITERBI type prediction sequence
	 */
	BRNN_VITERBI(EnumSet.of(SequenceAttributes.PREDICTION)),
	/**
	 * HMM21 Multiple sequence type prediction
	 */
	HMM21_MULTISEQ(EnumSet.of(SequenceAttributes.PREDICTION)),
	/**
	 * HMM21 Uni-sequence type prediction
	 */
	HMM21_UNISEQ(EnumSet.of(SequenceAttributes.PREDICTION)),
	/**
	 * HMM75 Multiple sequence type prediction
	 */
	HMM75_MULTISEQ(EnumSet.of(SequenceAttributes.PREDICTION)),
	/**
	 * HMM75 Uni-sequence type prediction
	 */
	HMM75_UNISEQ(EnumSet.of(SequenceAttributes.PREDICTION)),

	/**
	 * Observation type used for testing
	 */
	TEST_OBSERVATION(EnumSet.of(SequenceAttributes.OBSERVATION)),

	/**
	 * Prediction type used for testing
	 */
	TEST_PREDICTION(EnumSet.of(SequenceAttributes.PREDICTION));

	private enum SequenceAttributes {
		PREDICTION, OBSERVATION, EXTENDED_CHARS
	}

	private final EnumSet<SequenceAttributes> attributes;

	/**
	 * Set of all available prediction types
	 */
	public static final EnumSet<SequenceType> PREDICTION_TYPES = getByAttribute(SequenceAttributes.PREDICTION);

	/**
	 * Set of all avalable observation types
	 */
	public static final EnumSet<SequenceType> OBSERVATION_TYPES = getByAttribute(SequenceAttributes.OBSERVATION);

	private SequenceType(final EnumSet<SequenceAttributes> attributes) {
		this.attributes = attributes;
	}

	private SequenceType() {
		this(EnumSet.noneOf(SequenceAttributes.class));
	}

	/**
	 * Returns true if this is a prediction type
	 * 
	 * @return true if prediction type
	 */
	public boolean isPrediction() {
		return attributes.contains(SequenceAttributes.PREDICTION);
	}

	/**
	 * Returns true if this is a observation type
	 * 
	 * @return true if observation type
	 */
	public boolean isObservation() {
		return attributes.contains(SequenceAttributes.OBSERVATION);
	}

	/**
	 * Returns true if this uses 8 characters instead of 3.
	 * 
	 * @return true if has extended characters
	 */
	public boolean hasExtendedChars() {
		return attributes.contains(SequenceAttributes.EXTENDED_CHARS);
	}

	/**
	 * Return true if primary sequence
	 * 
	 * @return true if primary
	 */
	public boolean isPrimary() {
		return this == SequenceType.PRIMARY;
	}

	private static EnumSet<SequenceType> getByAttribute(final SequenceAttributes attribute) {
		final EnumSet<SequenceType> set = EnumSet.noneOf(SequenceType.class);
		for (final SequenceType type : SequenceType.values()) {
			if (type.attributes.contains(attribute)) {
				set.add(type);
			}
		}
		return set;
	}
}