package dk.sdu.imada.jkkn04.Protein.gui;

import java.awt.Component;
import java.awt.Container;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.net.URI;
import java.net.URL;
import java.util.Iterator;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.plaf.ScrollBarUI;

import org.swixml.ConverterLibrary;
import org.swixml.SwingEngine;
import org.swixml.SwingTagLibrary;

import dk.jkkn.util.swixml.ClassNameConverter;
import dk.jkkn.util.swixml.StringArrayConverter;

/**
 * @author Kristian Krmmer Nielsen <jkkn@jkkn.dk>
 */
public abstract class ProteinWindow {

	static private final String HOMEPAGE = "http://jkkn.dk/sdu/proteins/";

	private final JFrame mainFrame;

	JPanel labelPaneForSeqs, gridForSeqs;
	JScrollPane scrollPaneForSeqs;

	/* Actions */

	/**
	 * Action for exit menu item
	 */
	public Action actionExit = new AbstractAction() {
		private static final long serialVersionUID = 1L;

		public void actionPerformed(final ActionEvent e) {
			System.exit(0);
		}
	};

	/**
	 * Action for help menu item (opens homepage)
	 */
	public Action actionHelp = new AbstractAction() {
		private static final long serialVersionUID = 1L;

		public void actionPerformed(final ActionEvent e) {
			boolean okay = false;
			try {
				if (Desktop.isDesktopSupported()) {
					final Desktop desktop = Desktop.getDesktop();
					if (desktop.isSupported(Desktop.Action.BROWSE)) {
							desktop.browse(new URI(HOMEPAGE));
							okay = true;
					}
				}
			} catch (final Throwable th) {
				th.printStackTrace();
			}
			if (!okay) {
				JOptionPane.showMessageDialog(getMainFrame(),
						"You can find the homepage at: \n" + HOMEPAGE, getMainFrame()
								.getTitle(), JOptionPane.INFORMATION_MESSAGE);
			}
		}
	};

	/**
	 * Action for opening about dialog
	 */
	public Action actionAbout = new AbstractAction() {
		private static final long serialVersionUID = 1L;

		public void actionPerformed(final ActionEvent e) {
			JDialog dialog;
			try {
				dialog = (JDialog) renderWindow(ProteinWindow.class
						.getResource("AboutWindow.xml"));
				dialog.setModal(true);
				dialog.pack();
				dialog.setLocationRelativeTo(getMainFrame());
				dialog.setVisible(true);
			} catch (final Exception e1) {
				e1.printStackTrace();
			}
		}
	};

	/**
	 * Action for closing about dialog
	 */
	public Action actionCloseAbout = new AbstractAction() {
		private static final long serialVersionUID = 1L;

		public void actionPerformed(final ActionEvent e) {
			SwingUtilities.getWindowAncestor((Component) e.getSource()).dispose();
		}
	};

	/**
	 * Creates a new Protein GUI Window
	 * 
	 * @throws Exception in case of parse error
	 */
	public ProteinWindow() throws Exception {

		ConverterLibrary.getInstance().register(String[].class,
				new StringArrayConverter());
		ConverterLibrary.getInstance().register(Class.class, new ClassNameConverter());
		SwingTagLibrary.getInstance().registerTag("resulttable", ResultTable.class);

		final Container container = renderWindow(ProteinWindow.class
				.getResource("ProteinWindow.xml"));
		mainFrame = (JFrame) container;
		SwingEngine.setAppFrame(getMainFrame());

		// Manually we need to set the correct border needed for the horizontal
		// scrollbar
		final JScrollBar sc = scrollPaneForSeqs.getHorizontalScrollBar();
		final ScrollBarUI scrollUI = sc.getUI();
		final int scrollHeight = (int) scrollUI.getPreferredSize(sc).getHeight();
		labelPaneForSeqs.setBorder(new EmptyBorder(0, 0, scrollHeight, 0));
		scrollPaneForSeqs
				.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrollPaneForSeqs
				.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
		// Synchronizes size of grid-fields (depends upon fonts used)
		final int height = Math.max(
				labelPaneForSeqs.getComponent(3).getPreferredSize().height, gridForSeqs
						.getComponent(0).getPreferredSize().height);

		labelPaneForSeqs.getComponent(0).setMinimumSize(new Dimension(1, height));
		labelPaneForSeqs.getComponent(0).setMaximumSize(new Dimension(1, height));
		labelPaneForSeqs.getComponent(0).setPreferredSize(new Dimension(1, height));

		mainFrame.setLocationRelativeTo(null);
	}

	/**
	 * Method for rendering a window, adds support for bold and text with increased
	 * fontsize
	 * 
	 * @param swixml SWIXML resource
	 * @return Container for dialog/frame
	 * @throws Exception In case of any parsing errors
	 */
	Container renderWindow(final URL swixml) throws Exception {

		final SwingEngine swix = new SwingEngine(this);
		final Container container = swix.render(swixml);

		// Make headers bold
		final Iterator<?> i = swix.getAllComponentItertor();
		while (i.hasNext()) {
			final Object o = i.next();
			if (o instanceof JComponent) {
				final JComponent c = (JComponent) o;
				if (c.getClientProperty("makeBold") != null) {
					if (c.getClientProperty("makeBold").equals("true")) {
						c.setFont(c.getFont().deriveFont(Font.BOLD));
					}
					c.putClientProperty("makeBold", null);
				}
				if (c.getClientProperty("incrFont") != null) {
					try {
						c.setFont(c.getFont().deriveFont(
								c.getFont().getSize()
										+ Float.parseFloat((String) c
												.getClientProperty("incrFont"))));
					} catch (final NumberFormatException nfe) {
						nfe.printStackTrace();
					} finally {
						c.putClientProperty("incrFont", null);
					}
				}
			}
		}
		return container;
	}

	/**
	 * Returns frame for main window
	 * 
	 * @return frame
	 */
	public JFrame getMainFrame() {
		return mainFrame;
	}

}
