package dk.sdu.imada.jkkn04.Protein.tests;

import java.text.CharacterIterator;
import java.text.StringCharacterIterator;
import java.util.Map;

/**
 * Implementation of the MCC evaluation
 * 
 * @author Kristian Krmmer Nielsen <jkkn@jkkn.dk>
 */
public class MCC extends AbstractEvaluation {

	private double[] MCC = new double[State.COUNT];
	private int sequenceLength;

	/**
	 * Performs the MCC evaluations
	 * 
	 * @see dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation#performEvaluation()
	 */
	@Override
	public void performEvaluation() {
		final String seqObserved = observedProtein.getSequenceWithoutUnknowns();
		final String seqPredict = predictedProtein.getSequenceWithoutUnknowns();

		if (seqObserved.length() != seqPredict.length())
			// FIXME: throw something
			return;
		sequenceLength = seqObserved.length();

		final CharacterIterator iObserved = new StringCharacterIterator(seqObserved);
		final CharacterIterator iPredict = new StringCharacterIterator(seqPredict);

		final int tp[] = new int[State.COUNT]; // True Positive
		final int tn[] = new int[State.COUNT]; // True Negative
		final int fp[] = new int[State.COUNT]; // False Positive
		final int fn[] = new int[State.COUNT]; // False Negative

		char chrObserved = iObserved.current();
		char chrPredict = iPredict.current();
		while (chrObserved != CharacterIterator.DONE
				&& chrPredict != CharacterIterator.DONE) {
			for (final State state : State.values()) {
				final int c = state.getInt();
				final char chr = state.getChar();
				if (chrObserved == chr) {
					if (chrPredict == chr) {
						tp[c]++;
					} else {
						fn[c]++;
					}
				} else {
					if (chrPredict == chr) {
						fp[c]++;
					} else {
						tn[c]++;
					}
				}
			}

			chrObserved = iObserved.next();
			chrPredict = iPredict.next();
		}

		// Summarize per state
		for (final State state : State.values()) {
			final int c = state.getInt();

			final double divisor = Math.sqrt((tp[c] + fn[c]) * (tn[c] + fp[c])
					* (tp[c] + fp[c]) * (tn[c] + fn[c]));
			double result = 0;
			// If denominator is zero, we will return zero:
			if (divisor != 0) {
				result = (tp[c] * tn[c] - fp[c] * fn[c]) / divisor;
			}

			MCC[c] = result;

		}

	}

	/**
	 * Return results from the evaluations, MCC-H, MCC-E, MCC-C, length
	 * 
	 * @see dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation#getResults()
	 */
	@Override
	public Map<String, Object> getResults() {
		final Map<String, Object> results = super.getResults();

		for (final State state : State.values()) {
			final int c = state.getInt();
			results.put("MCC-" + state.name(), MCC[c] / sequences);
		}

		results.put("length", sequenceLength);
		return results;
	}

	/**
	 * Sums with another MCC evaluation
	 * 
	 * @see dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation#sumWith(dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation)
	 */
	@Override
	public void sumWith(final AbstractEvaluation evaluation) {
		super.sumWith(evaluation);
		if (evaluation != null) {
			if (!(evaluation instanceof MCC)) {
				// FIXME: throw something
			} else {
				final MCC other = (MCC) evaluation;
				for (final State state : State.values()) {
					final int c = state.getInt();
					MCC[c] += other.MCC[c];
				}
				sequenceLength += other.sequenceLength;
			}
		}
	}

	/**
	 * Clones the MCC evaluator
	 * 
	 * @see dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation#clone()
	 */
	@Override
	public MCC clone() {
		final MCC mcc = (MCC) super.clone();
		mcc.MCC = mcc.MCC.clone();
		return mcc;
	}

}
