package dk.sdu.imada.jkkn04.Protein.tests;

import java.text.CharacterIterator;
import java.text.StringCharacterIterator;
import java.util.Map;

/**
 * Implemenntation of the class QScore evalution of protein sequences.
 * 
 * @author Kristian Krmmer Nielsen <jkkn@jkkn.dk>
 */
public class QScore extends AbstractEvaluation {

	private int resultMatrix[][] = new int[State.COUNT][State.COUNT];
	private int opsTotal[] = new int[State.COUNT];
	private int totalSum;
	private int sequenceLength;

	/**
	 * Perform the QScore calcuation.
	 * 
	 * @see dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation#performEvaluation()
	 */
	@Override
	public void performEvaluation() {
		final String seqObserved = observedProtein.getCleanSequence();
		final String seqPredict = predictedProtein.getCleanSequence();

		if (seqObserved.length() != seqPredict.length())
			// FIXME: throw something
			return;
		sequenceLength = seqObserved.length();

		final CharacterIterator iObserved = new StringCharacterIterator(seqObserved);
		final CharacterIterator iPredict = new StringCharacterIterator(seqPredict);

		resultMatrix = new int[State.COUNT][State.COUNT];
		opsTotal = new int[State.COUNT];
		totalSum = 0;
		char chrObserved = iObserved.current();
		char chrPredict = iPredict.current();
		while (chrObserved != CharacterIterator.DONE
				&& chrPredict != CharacterIterator.DONE) {
			final int intObserved = mapCharToInt(chrObserved);
			final int intPredict = mapCharToInt(chrPredict);

			if (intObserved != -1) {
				if (intPredict != -1) {
					resultMatrix[intObserved][intPredict]++;
					if (intObserved == intPredict) {
						totalSum++;
					}
				}
				opsTotal[intObserved]++;
			}

			chrObserved = iObserved.next();
			chrPredict = iPredict.next();
		}

	}

	/**
	 * Returns the results of the QScore-evaluation. We return both a matrix: Q-HH, Q-HE,
	 * Q-HC, Q-EH,.... And a accumulated: Q-H, Q-E, Q-C And a total: Q3
	 * 
	 * @see dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation#getResults()
	 */
	@Override
	public Map<String, Object> getResults() {
		final Map<String, Object> results = super.getResults();

		for (final State iState : State.values()) {
			final int i = iState.getInt();
			for (final State jState : State.values()) {
				final int j = jState.getInt();
				results.put("Q-" + iState.name() + jState.name(), (opsTotal[i] == 0 ? 0
						: ((double) resultMatrix[i][j] / opsTotal[i] * 100)));
			}
			/* also adding diagonal */
			results.put("Q-" + iState.name(), (opsTotal[i] == 0 ? 0
					: ((double) resultMatrix[i][i] / opsTotal[i] * 100)));
		}
		// Add total sum to results
		results.put("Q3", (sequenceLength == 0 ? 0
				: ((double) totalSum / sequenceLength * 100)));

		results.put("length", sequenceLength);

		return results;
	}

	/**
	 * Sums a QScore result with another QScore result
	 * 
	 * @see dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation#sumWith(dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation)
	 */
	@Override
	public void sumWith(final AbstractEvaluation evaluation) {
		super.sumWith(evaluation);
		if (evaluation != null) {
			if (!(evaluation instanceof QScore)) {
				// FIXME: throw something
			} else {
				final QScore other = (QScore) evaluation;
				for (final State state : State.values()) {
					final int i = state.getInt();

					// Sum result matrix
					for (final State stateJ : State.values()) {
						final int j = stateJ.getInt();
						resultMatrix[i][j] += other.resultMatrix[i][j];
					}

					opsTotal[i] += other.opsTotal[i];
				}
				totalSum += other.totalSum;
				sequenceLength += other.sequenceLength;
			}
		}
	}

	/**
	 * Clones the QScore evaluation result.
	 * 
	 * @see dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation#clone()
	 */
	@Override
	public QScore clone() {
		final QScore qscore = (QScore) super.clone();
		qscore.opsTotal = qscore.opsTotal.clone();
		qscore.resultMatrix = qscore.resultMatrix.clone();
		return qscore;
	}

}
