package dk.sdu.imada.jkkn04.Protein.tests;

import java.text.CharacterIterator;
import java.text.StringCharacterIterator;
import java.util.Map;

/**
 * SOV (Segment OVerlap) measure Secondary structure prediction accuracy
 * evaluation.<br/><br/>
 * 
 * This is a port of the original program posted and written by Adam
 * Zemla (11/16/1996)<br/>
 * 
 * It was posted on http://as2ts.llnl.gov/AS2TS/download_area/<br/><br/>
 * And originally written in C
 * 
 * @author Kristian Krmmer Nielsen <jkkn@jkkn.dk>
 */
public class SOV extends AbstractEvaluation {

	private static final boolean USE_SOV_1994_JMB_METHOD = false;
	private static final double SOV_DELTA_S = 0.5;
	private static final double SOV_DELTA = 1.0;

	private double resultsSOV[] = new double[State.COUNT]; // results by H, E, C
	private double totalSOV = 0;
	private int sequenceLength = 0;

	/**
	 * Performs the actual SOV calculation.
	 * 
	 * @see dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation#performEvaluation()
	 */
	@Override
	public void performEvaluation() {
		final String seqObserved = observedProtein.getCleanSequence();
		final String seqPredict = predictedProtein.getCleanSequence();

		// Split by segments of unknown
		final String[] segObserved = seqObserved.split("X+");
		final String[] segPredict = seqPredict.split("X+");

		if (segObserved.length != segPredict.length)
			// FIXME: throw something
			return;

		resultsSOV = new double[State.COUNT];
		totalSOV = 0;
		int totalLength = 0;

		for (int i = 0; i < segObserved.length; i++) {
			if (segObserved[i].length() != segPredict[i].length())
				// FIXME: throw something
				return;
			final int len = segPredict[i].length();

			for (final State state : State.values()) {
				resultsSOV[state.getInt()] += calcSOVPart(segPredict[i], segObserved[i],
						state.getChar())
						* len;
			}

			totalSOV += calcSOVPart(segPredict[i], segObserved[i]) * len;

			totalLength += len;
		}

		if (totalLength > 0) {
			for (final State state : State.values()) {
				resultsSOV[state.getInt()] /= totalLength;
			}
			totalSOV /= totalLength;
		}

		sequenceLength = totalLength;

	}

	private double calcSOVPart(final String seqPredict, final String seqObserved) {
		return calcSOVPart(seqPredict, seqObserved, null);
	}

	private double calcSOVPart(final String seqPredict, final String seqObserved,
			final Character stateChar) {
		final CharacterIterator iPredict = new StringCharacterIterator(seqPredict);
		final CharacterIterator iObserved = new StringCharacterIterator(seqObserved);

		double out = 0;
		double s = 0;

		char chrObserved = iObserved.first();
		int totalResidues = 0;

		while (chrObserved != CharacterIterator.DONE) {
			if (stateChar == null || chrObserved == stateChar) {
				totalResidues++;
			}
			chrObserved = iObserved.next();
		}

		chrObserved = iObserved.first();
		while (chrObserved != CharacterIterator.DONE) {
			final int beginSegment1 = iObserved.getIndex();
			final char s1 = chrObserved;
			do {
				chrObserved = iObserved.next();
			} while (chrObserved != CharacterIterator.DONE && s1 == chrObserved);
			final int endSegment1 = iObserved.getIndex() - 1;
			final int lengthSegment1 = endSegment1 - beginSegment1 + 1;

			int multiple = 0;
			char chrPredict = iPredict.first();

			while (chrPredict != CharacterIterator.DONE) {
				final int beginSegment2 = iPredict.getIndex();
				final char s2 = chrPredict;
				do {
					chrPredict = iPredict.next();
				} while (chrPredict != CharacterIterator.DONE && s2 == chrPredict);
				final int endSegment2 = iPredict.getIndex() - 1;
				final int lengthSegment2 = endSegment2 - beginSegment2 + 1;

				if (stateChar == null || s1 == stateChar) {
					if (s1 == s2 && endSegment2 >= beginSegment1
							&& beginSegment2 <= endSegment1) {
						if (multiple > 0 && !USE_SOV_1994_JMB_METHOD) {
							totalResidues = totalResidues + lengthSegment1;
						}
						multiple++;
						int j1, j2, k1, k2;
						// Place first segment start offset in j1
						if (beginSegment1 > beginSegment2) {
							j1 = beginSegment1;
							j2 = beginSegment2;
						} else {
							j1 = beginSegment2;
							j2 = beginSegment1;
						}
						// Place last segment end offset in k2
						if (endSegment1 < endSegment2) {
							k1 = endSegment1;
							k2 = endSegment2;
						} else {
							k1 = endSegment2;
							k2 = endSegment1;
						}
						int d;
						final int minov = k1 - j1 + 1;
						final int maxov = k2 - j2 + 1;
						final int d1 = (int) Math.floor(lengthSegment1 * SOV_DELTA_S);
						final int d2 = (int) Math.floor(lengthSegment2 * SOV_DELTA_S);
						if (d1 <= d2 || USE_SOV_1994_JMB_METHOD) {
							d = d1;
						} else {
							d = d2;
						}
						if (d > minov) {
							d = minov;
						}
						if (d > maxov - minov) {
							d = maxov - minov;
						}
						double x = SOV_DELTA * d;
						x = (minov + x) * lengthSegment1;
						if (maxov > 0) {
							s += x / maxov;
						} else {
							System.err
									.println(String
											.format(
													"\n ERROR! minov = %-4d maxov = %-4d length = %-4d d = %-4d   %4d %4d  %4d %4d",
													minov, maxov, lengthSegment1, d,
													beginSegment1 + 1, endSegment1 + 1,
													beginSegment2 + 1, endSegment1 + 1));
							// FIXME: throw something
						}
					}
				}
			}
		}
		if (totalResidues > 0) {
			out = s / totalResidues;
		} else {
			out = 1.0;
		}
		return out * 100;
	}

	/**
	 * Returns result of the SOV calculation We return SOV-H, SOV-E, SOV-C and a total
	 * SOV3
	 * 
	 * @see dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation#getResults()
	 */
	@Override
	public Map<String, Object> getResults() {
		final Map<String, Object> results = super.getResults();

		for (final State state : State.values()) {
			final int c = state.getInt();
			results.put("SOV-" + state.name(), resultsSOV[c] / sequences);
		}
		results.put("SOV3", totalSOV / sequences);
		results.put("length", sequenceLength);

		return results;
	}

	/**
	 * Summerize with another SOV result.
	 * 
	 * @see dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation#sumWith(dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation)
	 */
	@Override
	public void sumWith(final AbstractEvaluation evaluation) {
		super.sumWith(evaluation);
		if (evaluation != null) {
			if (!(evaluation instanceof SOV)) {
				// FIXME: throw something
			} else {
				final SOV other = (SOV) evaluation;
				for (final State state : State.values()) {
					final int c = state.getInt();
					resultsSOV[c] += other.resultsSOV[c];
				}
				totalSOV += other.totalSOV;
				sequenceLength += other.sequenceLength;
			}
		}
	}

	/**
	 * Clone the SOV result
	 * 
	 * @see dk.sdu.imada.jkkn04.Protein.tests.AbstractEvaluation#clone()
	 */
	@Override
	public AbstractEvaluation clone() {
		final SOV sov = (SOV) super.clone();
		sov.resultsSOV = sov.resultsSOV.clone();
		return sov;
	}

}
